$LOAD_PATH.unshift __dir__

# stdlib
require 'optparse'
require 'net/http'
require 'json'

# internal
require 'oak/version'
require 'oak/monkey'
require 'oak/easter_egg'

module Oak
  def self.go(args)
    Oak::Request.new(args).go
  end

  class Request
    DEFAULT_HEADERS = {'authorization' => ENV['OAK_API_KEY']}

    class HTTP
      attr_reader :verb, :uri, :headers, :body

      VERBS = { g: :get, p: :post, c: :patch, d: :delete }

      def initialize(args, base_url, api_version, headers, debug=nil)
        @verb = args.shift.to_sym
        @uri = URI("#{base_url}/api/#{api_version}/#{args.shift}")
        @body = args.shift
        @verb = VERBS[@verb] || @verb
        @headers = headers
        raise UnknownHTTPVerb unless VERBS.values.include?(@verb)
      end

      def set_request
        @req = Net::HTTP.const_get(verb.to_s.capitalize).new(uri, headers)
        @req.body = body
        @req
      end
    end

    attr_reader :options, :base_url, :args, :debug, :headers

    def initialize(args)
      @options = Options.parse(args)
      @args = args
      @base_url = development? ? 'http://localhost:3000' : 'https://oak.software'
      @options[:headers] = @options[:headers].to_h.merge({'authorization' => ENV['OAK_API_KEY_DEVEL']}) if development?
      @headers = DEFAULT_HEADERS.merge(@options[:headers].to_h)
      @debug = @options[:debug]
      info self if @debug
    end

    def development?
      @options[:environment] == 'development'
    end

    def go
      if @options[:list]
        args.unshift(?g, "endpoints")
      end

      req = HTTP.new(args, base_url, options[:api], headers, debug).set_request

      info [req, req.uri, req.uri.host, req.uri.port, req.body] if debug
      info req.to_hash if debug

      http = Net::HTTP.new(req.uri.host, req.uri.port)
      http.use_ssl = true unless development?
      res = http.request(req)

      puts res.body
    end

    class Options
      def self.parse(args)
        options = {}

        OptionParser.new do |opts|
          opts.program_name = 'oak-api'
          opts.banner = "Usage: #{opts.program_name} [options] [http_verb] [endpoint] [http_body]"

          opts.separator ""
          opts.separator "Options:"

          opts.on("-d", "--debug", "Enable debug mode") do |v|
            options[:debug] = v
          end

          opts.on("-e", "--environment=ENV", "Enable debug mode ") do |v|
            options[:environment] = v
          end

          opts.on("-l", "--list", "List API endpoints") do |v=x|
            options[:list] = v
          end

          opts.on_tail("-a", "--api=VERSION", "Target a specific API version (defaults to latest)") do |v|
            options[:api] = v
          end

          opts.on_tail("-H", "--headers=HEADERS", "HEADERS is a JSON string with HTTP headers") do |v|
            options[:headers] = JSON.parse(v)
          end

          opts.on("-o", "--output=NAME", "Specify output file name") do |v|
            options[:output] = v
          end

          opts.on("-t", "--tree", "Print a welcome message") do |v|
            Oak.greetings
            abort
          end

          opts.on_tail("-h", "--help", "Print help", "something else") do |v|
            abort opts.to_s
          end

          opts.on_tail("-v", "--version", "Print version") do |v|
            abort "#{opts.program_name} #{VERSION}"
          end
        end.parse!(args)

        options
      end
    end
  end
end
